/*jslint white: true, onevar: true, browser: false, evil: true, undef: true, nomen: true, regexp: true, plusplus: true, bitwise: true, newcap: true, maxerr: 50, indent: 4 */

// Standard setup
this.name           = "Factions";
this.author         = "m4r35n357";
this.copyright      = "© 2011, 2012 m4r35n357 (Ian Smith), Creative Commons: attribution, non-commercial, sharealike.";
this.description    = "Space battles for MultiGovernment systems";
this.version        = "1.12";

this.startUp = function () {
  // Logging
  this.$logging = false;

  // AI debug
  this.$aiDebug = false;

  // uber test mode
  this.$uberTest = false;

  // Battles at all systems
  this.$always = false;

  // Battle location
  this.$home = Vector3D(0.0, 0.0, 0.5).fromCoordinateSystem("wpu");

  // Each faction starts this fraction of w->p away from the site
  this.$start = 0.3;

  // Cluster radius for spawning factions
  this.$spread = 5000;

  // Battle probability
  this.$battleProbability = 0.25;

  // All factions combined, but not including escorts
  this.$totalShips = 48;

  // The cast
  this.$factions = [];
  this.$factions.push({
    title: "Communist",
    role: "factions-faction1",
    unit: Vector3D(0.0, -1.0, 0.0).fromCoordinateSystem("wpu"),
    scanColour: "1.0 0.0 0.5", // pinkish
  });
  this.$factions.push({
    title: "Dictatorship",
    role: "factions-faction2",
    unit: Vector3D(0.0, 1.0, 0.0).fromCoordinateSystem("wpu"),
    scanColour: "1.0 0.5 0.0", // orangeish
  });
  this.$factions.push({
    title: "Feudal",
    role: "factions-faction3",
    unit: Vector3D(-1.0, 0.0, 0.0).fromCoordinateSystem("wpu"),
    scanColour: "0.0 0.5 1.0", // blueish
  });
  this.$factions.push({
    title: "Local Militia",
    role: "factions-faction4",
    unit: Vector3D(1.0, 0.0, 0.0).fromCoordinateSystem("wpu"),
    scanColour: "0.0 1.0 0.5", // turquoiseish
  });
/*
  this.$factions.push({
    role: "factions-faction5",
    unit: Vector3D(0.0, 0.0, 1.0).fromCoordinateSystem("wpu"),
    scanColour: "0.5 0.0 1.0",  // purpleish
  });
  this.$factions.push({
    role: "factions-faction6",
    unit: Vector3D(0.0, 0.0, -1.0).fromCoordinateSystem("wpu"),
    scanColour: "0.5 1.0 0.0", // greenish
  });
*/

  this.$enemyVar = "factions_enemy_";
};

// Helpers
this.$setDisplayName = function (ship) {
  if (this.$logging) {
    return ship.$fighterName + " " + ship.name;
  } else {
    return ship.name;
  }
};

this.$addFaction = function (faction, shipList) {
  for (var i = 0; i < shipList.length; ++i) {
    shipList[i].$faction = faction;
    shipList[i].$fighterName = faction.title + " " + i;
    shipList[i].displayName = this.$setDisplayName(shipList[i]);
    shipList[i].bounty = 100.0;
    shipList[i].savedCoordinates = this.$home;
    shipList[i].scannerDisplayColor1 = "darkGrayColor";
    shipList[i].scannerDisplayColor2 = faction.scanColour;
    shipList[i].switchAI("factionAI.plist");
    shipList[i].setScript("faction.js");
    this.$logging && log(this.name, faction.title + " [" + i + "] " + shipList[i].name + ", " + shipList[i].AI + " " + shipList[i].script);
    this.$aiDebug && (shipList[i].reportAIMessages = true);
    if (shipList[i].escorts) {
      for (var j = 0; j < shipList[i].escorts.length; ++j) {
        shipList[i].escorts[j].$faction = faction;
        shipList[i].escorts[j].$isEscort = true;
        shipList[i].escorts[j].$fighterName = faction.title + " " + i + ":" + j;
        shipList[i].escorts[j].displayName = shipList[i].escorts[j].$fighterName + " " + shipList[i].escorts[j].name;
        shipList[i].escorts[j].bounty = 50.0;
        shipList[i].escorts[j].savedCoordinates = this.$home;
        shipList[i].escorts[j].scannerDisplayColor1 = "darkGrayColor";
        shipList[i].escorts[j].scannerDisplayColor2 = faction.scanColour;
        shipList[i].escorts[j].switchAI("battleEscortAI.plist");
        this.$logging && log(this.name, "  [" + i + ":" + j + "] " + shipList[i].escorts[j].name + ", " + shipList[i].escorts[j].AI + " " + shipList[i].escorts[j].script);
        this.$aiDebug && (shipList[i].escorts[j].reportAIMessages = true);
      }
    }
  }
};

this.$writeMissionVariables = function () {
  for (var i = 0; i < this.$fightingFactions; ++i) {
    if (this.$factions[i].playerHasKilled) {
      missionVariables[this.$enemyVar + i] = 1;
    } else {
      missionVariables[this.$enemyVar + i] = 0;
    }
  }
};

// Event handlers
this.shipWillExitWitchspace = function () {
  // Activation is based on the system government type being Multigovernment
  if (this.$always || this.$uberTest || (!system.isInterstellarSpace && (system.government === 2) && (Math.random() <= this.$battleProbability))) {
    if (this.$uberTest) {
      // celebrity deathmatch mode (remember to copy shipdata.plist!)
      this.$home = Vector3D(0.0, 0.0, 0.05).fromCoordinateSystem("wpu");
      this.$start = 0.05;
      this.$fightingFactions = 2;
      this.$totalShips = 24;
    } else {
      //only the hard-core turn up at higher tech levels, GalCop cares more there too
      switch (system.techLevel + 1) {
        case 1: case 2: case 3: case 4: case 5: {
          this.$fightingFactions = this.$factions.length;
          this.$nCops = 1;
        }
        break;
        case 6: case 7: case 8: case 9: {
          this.$fightingFactions = this.$factions.length - 1;
          this.$nCops = 2;
        }
        break;
        case 10: case 11: case 12: case 13: case 14: case 15: { // yes, I know 12 is the highest TL multigov ;)  see uber-mode
          this.$fightingFactions = this.$factions.length - 2;
          this.$nCops = 3;
        }
        break;
        default: {
          this.$logging && log(this.name, "ERROR: Tech Level out of range!");  // should never happen!
        }
      }
      // Galcop
      this.$police = system.addShips("factions-galcop", this.$nCops, Vector3D(0.0, 0.0, 0.0).fromCoordinateSystem("wpu"), this.$spread * 2.0);
      for (var i = 0; i < this.$police.length; ++i) {
        this.$police[i].$fighterName = "GalCop " + i;
        this.$police[i].displayName = this.$setDisplayName(this.$police[i]);
        this.$logging && log(this.name, "GalCop [" + i + "] " + this.$police[i].name + ", " + this.$police[i].AI + " " + this.$police[i].script);
      }
    }
    // Factions
    this.$factionSize = this.$totalShips / this.$fightingFactions;
    this.$logging && log(this.name, "Battle: " + this.$fightingFactions + " factions, " + this.$factionSize + " ships per faction at " + system.name);
    for (var i = 0; i < this.$fightingFactions; ++i) {
      this.$addFaction(this.$factions[i], system.addShips(this.$factions[i].role, this.$factionSize, this.$home.add(this.$factions[i].unit.multiply(this.$start)), this.$spread));
      if (missionVariables[this.$enemyVar + i] && (missionVariables[this.$enemyVar + i] > 0)) {
        this.$factions[i].playerIsEnemy = true;
      } else {
        this.$factions[i].playerIsEnemy = false;
      }
      this.$factions[i].playerHasKilled = false;
    }
  }
};

this.shipDockedWithStation = function (station) {
  this.$writeMissionVariables();
};

this.shipWillEnterWitchspace = function (cause) {
  this.$writeMissionVariables();
};

