/*jslint white: true, onevar: true, browser: false, evil: true, undef: true, nomen: true, regexp: true, plusplus: true, bitwise: true, newcap: true, maxerr: 50, indent: 4 */

this.name        = "Faction"; 
this.author      = "m4r35n357";
this.copyright   = "© 2011 m4r35n357 (Ian Smith), Creative Commons: attribution, non-commercial, sharealike.";
this.description = "Faction ship script"; 
this.version     = "1.12";

// Logging
this.$logging = false;

this.$extraLogging = false;

// a dependency!
this.$ccl = new worldScripts.Cabal_Common_Functions.Cabal_Common();

// initialize
this.$escortsAttacking = false;

// helpers
this.$longName = function (ship) {
  if (ship && ship.isValid) {
    if (ship === player.ship) {
      return "*PLAYER*";
    } else {
      return ship.displayName;
    }
  } else {
    return "[DEAD SHIP]";
  }
};

this.$safeExitAI = function (ship) {
  while (ship.hasSuspendedAI) {
    ship.exitAI();
  }
}

this.$intercept = function (ship, whom) {
  ship.target = whom;
  ship.setAI("interceptAI.plist");
  ship.reactToAIMessage("ATTACK_SHIP");
}

this.$escortAttack = function (whom) {
  this.$logging && log(this.name, this.$longName(this.ship) + ": Escorts, attack " + this.$longName(whom));
  if (this.$escortsAttacking) {
    // already attacking, switch target
    for (var i = 0; i < this.ship.escorts.length; i += 1) {
      if (this.ship.escorts[i].target !== whom) {
        this.$logging && log(this.name, "  " + this.$longName(this.ship.escorts[i]) + " now attacking " + this.$longName(whom));
        this.$safeExitAI(this.ship.escorts[i]);
        this.$intercept(this.ship.escorts[i], whom);
      }
    }
  } else {
    // new target
    for (var i = 0; i < this.ship.escorts.length; i += 1) {
      this.$logging && log(this.name, "  " + this.$longName(this.ship.escorts[i]) + " attacking " + this.$longName(whom));
      this.$intercept(this.ship.escorts[i], whom);
    }
    this.$escortsAttacking = true;
  }
};

// AI extension callbacks
this.$scanForTarget = function () {
  var targetFilter = function (entity) {
    return (entity.$faction && (entity.$faction.title !== this.ship.$faction.title))
      || (entity.isPlayer && this.ship.$faction.playerIsEnemy)
      || (entity.isShip && ((entity.primaryRole === "factions-galcop") || (entity.isPolice)));
  };
  if (this.ship.position.distanceTo(this.ship.savedCoordinates) > 25600) {
    this.$logging && log(this.name, this.$longName(this.ship) + " out of range, returning to battle: ");
    this.ship.reactToAIMessage("NOTHING_FOUND");
  } else {
    var targetsInRange = system.filteredEntities(this, targetFilter, this.ship, this.ship.scannerRange);
    this.$logging && log(this.name, this.$longName(this.ship) + ": " + targetsInRange.length + "/" + system.filteredEntities(this, targetFilter).length + " in range");
    if (targetsInRange.length > 0) {
      // all ships
      if (this.ship.$faction.playerHasKilled) {
        this.ship.target = player.ship;
      } else {
        this.ship.target = targetsInRange[this.$ccl.rand(targetsInRange.length)];
      }
      this.$logging && log(this.name, this.$longName(this.ship) + " selected target: " + this.$longName(this.ship.target));
      this.ship.reactToAIMessage("TARGET_FOUND");
      this.shipTargetAcquired(this.ship.target);
      // ships with escorts
      if (this.ship.escorts && (this.ship.escorts.length > 0)) {
        this.$escortAttack(this.ship.target);
      }
    } else {
      this.$logging && log(this.name, this.$longName(this.ship) + " failed to find a target");
      this.ship.reactToAIMessage("NOTHING_FOUND");
    }
  }
};

this.$help = function () {
  this.$logging && log(this.name, this.$longName(this.ship) + ": Please help me, attack " + this.$longName(this.ship.target));
  if ((this.ship.target === player.ship) && !this.ship.$faction.playerHasKilled) {
    this.ship.commsMessage("Foe identified as " + player.name + ", piloting a " + player.ship.name + ".  Please assist!");
  }
  var buddies = system.filteredEntities(this, function (entity) { return entity.$faction && (entity.$faction.title === this.ship.$faction.title); });
  for (var i = 0; i < buddies.length; i += 1) {
    if (!buddies[i].$isEscort && !buddies[i].hasHostileTarget && !(buddies[i] === this.ship)) {
      buddies[i].target = this.ship.target;
      buddies[i].reactToAIMessage("TARGET_FOUND");
      this.$logging && log(this.name, this.$longName(buddies[i]) + " assisting . . . ");
    }
  }
}

this.$newState = function () {
  switch (this.ship.AIState) {
    case "ATTACK_SHIP": {
      this.$logging && log(this.name, this.$longName(this.ship) + " entering state: " + this.ship.AIState + ", target: " + this.$longName(this.ship.target));
    }
    break;
    case "FIND_ENEMIES": {
      this.$logging && log(this.name, this.$longName(this.ship) + " entering state: " + this.ship.AIState);
      this.ship.scannerDisplayColor1 = "lightGrayColor";
      // ships with escorts
      if (this.ship.escorts && (this.ship.escorts.length > 0)) {
        for (var i = 0; i < this.ship.escorts.length; i += 1) {
          this.ship.escorts[i].scannerDisplayColor1 = "lightGrayColor";
        }
      }
    }
    break;
    case "FLEE": {
      this.$logging && log(this.name, this.$longName(this.ship) + " entering state: " + this.ship.AIState);
      this.ship.scannerDisplayColor1 = "blackColor";
    }
    break;
    case "RETURN_TO_BATTLE": {
      this.$logging && log(this.name, this.$longName(this.ship) + " entering state: " + this.ship.AIState);
      this.ship.scannerDisplayColor1 = "darkGrayColor";
      // ships with escorts
      if (this.ship.escorts && (this.ship.escorts.length > 0)) {
        for (var i = 0; i < this.ship.escorts.length; i += 1) {
          this.ship.escorts[i].scannerDisplayColor1 = "darkGrayColor";
        }
      }
    }
    break;
  }
};

// Regular callbacks
this.shipBeingAttacked = function (whom) {
  // all ships
  if (whom && whom.$faction && (this.ship.$faction === whom.$faction)) {
    if (whom.$isEscort) {
      whom.reactToAIMessage("TARGET_LOST");
    } else {
      whom.reactToAIMessage("FRIENDLY_FIRE");
    }
    this.ship.reactToAIMessage("FRIENDLY_FIRE");
    this.$logging && log(this.name, "Friendly fire incident against " + this.$longName(this.ship) + " by " + this.$longName(whom) + "!");
  } else {
    this.shipTargetAcquired(whom);
    this.$extraLogging && log(this.name, this.$longName(this.ship) + " being attacked by " + this.$longName(whom));
    if ((whom === player.ship) && !this.ship.$faction.playerIsEnemy) {
      this.ship.$faction.playerIsEnemy = true;
      this.$logging && log(this.name, this.$longName(player.ship) + " is now an enemy of: " + this.ship.$faction.title);
      this.ship.commsMessage("Unknown foe detected, piloting a " + player.ship.name);
    }
    // ships with escorts
    if (this.ship.escorts && (this.ship.escorts.length > 0)) {
      this.$escortAttack(whom);
    }
  }
};

this.shipTargetAcquired = function (target) {
  // all ships
  if (target === player.ship) {
    this.ship.scannerDisplayColor1 = "redColor";
  } else {
    this.ship.scannerDisplayColor1 = "whiteColor";
  }
  // ships with escorts
  if (this.ship.escorts && (this.ship.escorts.length > 0)) {
    for (var i = 0; i < this.ship.escorts.length; i += 1) {
      if (target === player.ship) {
        this.ship.escorts[i].scannerDisplayColor1 = "redColor";
      } else {
        this.ship.escorts[i].scannerDisplayColor1 = "whiteColor";
      }
    }
  }
};

this.shipTargetLost = function (target) {
  // all ships
  this.ship.scannerDisplayColor1 = "lightGrayColor";
  // ships with escorts
  if (this.ship.escorts && (this.ship.escorts.length > 0)) {
    this.$logging && log(this.name, this.$longName(this.ship) + ": Escorts, heel!");
    if (this.$escortsAttacking) {
      for (var i = 0; i < this.ship.escorts.length; i += 1) {
        this.$logging && log(this.name, "  " + this.$longName(this.ship.escorts[i]) + " returning to escort duty");
        this.$safeExitAI(this.ship.escorts[i]);
      }
      this.$escortsAttacking = false;
    }
    for (var i = 0; i < this.ship.escorts.length; i += 1) {
      this.ship.escorts[i].scannerDisplayColor1 = "lightGrayColor";
    }
  }
};

this.shipDied = function (whom, why) {
  // all ships
  this.$logging && log(this.name, this.$longName(this.ship) + " killed by " + this.$longName(whom) + " (" + why + ")");
  if (whom === player.ship) {
    this.ship.$faction.playerHasKilled = true;
    this.$logging && log(this.name, this.$longName(player.ship) + " is now a mortal enemy of: " + this.ship.$faction.title);
    this.ship.commsMessage("Avenge me, destroy " + player.name + "!!!");
  }
  // ships with escorts
  if (this.ship.escorts && (this.ship.escorts.length > 0)) {
    for (var i = 0; i < this.ship.escorts.length; i += 1) {
      this.$logging && log(this.name, "  " + this.$longName(this.ship.escorts[i]) + " joining battle!");
      this.ship.$isEscort = false;
      this.ship.escorts[i].switchAI("factionAI.plist");
      this.ship.escorts[i].setScript("faction.js");
    }
  }
};

